//=============================================================================
// ANNAAntiCheat. (C) 2012 "Dei" & Nobody
//=============================================================================
/**
 Checks the consoleclass of a player.
 It's spawned by the engine after a player joined to check his/her client-initalized consoleclass.
 The check algorithm is nearly unbreakable because the server just requests
 the consoleclass information from the client and checks it server-side. (like miniMTL143 did it)
 However the difference to miniMTL143's check is, that miniMTL210 starts a timeout for the client to answer.
 So if some haxor hacked his copy of miniMTL210 to avert, that it sends his consoleclass information to the server,
 he will be kicked by the timeout.
 Also this check algorithm comes with a UID which is dynamically generated for the check session,
 so the haxor-client can't send a valid fake answer.

*/
class ANNAAntiCheat extends Actor;

replication
{
    reliable if(ROLE == ROLE_Authority)
       clientGiveInfo;

    reliable if(ROLE < ROLE_Authority)
       serverValidateInfo;
}

/** Saves the UID which is generated for the console-class check to validate the answer of the client. */
var private string checkUID;
var private string foundConsole;

/** @ignore */
var private bool bCheckPassed;
var private bool bCheatDetected;
var private bool bInitalized;

var string rootWindowList[3];

/**
 Starts a Timer of 5 seconds (to let the server time to send the just joined client all game relevant information)
 After that 5 seconds <i>Initalize()</i> function will be called to start the console check.
*/
simulated function PostBeginPlay()
{
  if(ANNAPlayer(Owner) != None)
  {
    if(level.NetMode != NM_StandAlone && ROLE == ROLE_Authority)
    {
        SetTimer(4, false);
    }
  }
}

/**
 Generating the UID, sending a check request to the client and starting the TimeOut
*/
simulated function Initalize()
{
    if(level.NetMode != NM_StandAlone && ROLE == ROLE_Authority)
    {
        if(ANNAPlayer(Owner) != none)
        {
            checkUID = class'ANNAManager'.static.generateUID();
            clientGiveInfo(checkUID, "Engine.Engine", "Engine.GameEngine", "Console", "GameEngine", "Root");
            bInitalized = True;
            SetTimer(4, false);
        }
    }
}

/**
 Sends a check request to the client, which will return the console class over <i>serverValidateConsole()</i>.
*/
simulated function clientGiveInfo(string checkID, string EngineSection, string GameEngineSection, string Console, string GameEngine, string Root)
{
  serverValidateInfo(ANNAPlayer(Owner).Player.Console.Class, string(ANNAPlayer(Owner).Player.Console.Class), GetConfig(EngineSection, Console), GetConfig(GameEngineSection, Console), GetConfig(EngineSection, GameEngine), ANNAPlayer(Owner).rootWindow.Class, string(ANNAPlayer(Owner).rootWindow.Class), GetConfig(EngineSection, Root), checkID);
}

/**
 Sends the answer with the requested information back to the server.
*/
function serverValidateInfo(Class<Console> consoleClass, string consoleClassString, string consoleString1, string consoleString2, string gameEngine, Class<RootWindow> rootClass, string rootClassString, string rootString, string checkID)
{
  local string conStr, rootStr;
  local bool bConsolePassed, bEnginePassed, bRootPassed;
  local int i;
  local class <RootWindow> RootWindow;

  if(checkID == checkUID)
  {
    if(consoleClass == Class'Engine.Console' && consoleClassString == "Engine.Console")
    {
      if((Len(consoleString1) > 0 && consoleString1 == "Engine.Console") || Len(consoleString1) == 0)
      {
        if((Len(consoleString2) > 0 && consoleString2 == "Class'Engine.Console'") || Len(consoleString2) == 0)
        {
          bConsolePassed = true;
        }
      }
    }

    if(!bConsolePassed)
      reportHack("Invalid console", consoleClass$"|"$consoleClassString$"|"$consoleString1$"|"$consoleString2);

    if(Len(gameEngine) > 0 && gameEngine == "DeusEx.DeusExGameEngine")
      bEnginePassed = true;
    else
      reportHack("Invalid engine", gameEngine);


    for(i = 0; i < 3; i++)
    {
      if(Len(rootWindowList[i]) > 0)
      {
        RootWindow = class<RootWindow>(DynamicLoadObject(rootWindowList[i], class'Class'));
        if(RootWindow != None && rootClass == RootWindow && string(rootClass) == rootClassString)
        {
          if((Len(rootString) > 0 && rootString == rootWindowList[i]) || Len(rootString) == 0)
          {
            bRootPassed = true;
            break;
          }
        }
      }
    }
    
    if(!bRootPassed)
      reportHack("Invalid rootwindow", rootString);

    if(bConsolePassed && bEnginePassed && bRootPassed)
      bCheckPassed = true;
  }
}

function reportHack(string cheatMethod, string cheatDetails)
{
  if(DeusExPlayer(Owner) != None)
  {
    Log(" ");
    Log("===============================");
    Log("         CHEAT FOUND           ");
    Log("-------------------------------");
    Log("Player Caught:   " $ DeusExPlayer(Owner).PlayerReplicationInfo.PlayerName $ ".");
    Log("Player Address:  " $ DeusExPlayer(Owner).GetPlayerNetworkAddress() $ ".");
    Log("Date:            " $ string(Level.Month) $ "/" $ string(Level.Day) $ "/" $ string(Level.Year) $ ".");
    Log("Time:            " $ string(Level.Hour) $ ":" $ string(Level.Minute) $ ":" $ string(Level.Second) $ ".");
    Log("Cheat Found:     " $ cheatMethod $ ".");
    Log("Cheat Details:   " $ cheatDetails $ ".");
    Log("===============================");
    Log(" ");
    DeusExPlayer(Owner).ForceDisconnect(cheatMethod$", disconnecting");
    DeusExPlayer(Owner).Destroy(); // Just in case..
  }
}

/**
 Is called by the engine, when the timeout has exceeded.
 if the server didn't get a valid answer yet, it will disconnect the client.
*/
simulated function Timer()
{
    if(DeusExPlayer(Owner) != None)
    {
        if(bInitalized)
        {
            if(!bCheckPassed)
            {
                DeusExPlayer(Owner).ForceDisconnect("Failed running AntiCheat, disconnecting");
                DeusExPlayer(Owner).Destroy(); // Just in case..
            }
            else
            {
                bInitalized = false;
                SetTimer(90+Rand(30), false);
            }
        }
        else
        {
            Initalize();
        }
    }
    else
      Destroy();
}

DefaultProperties
{
    bHidden=true
    rootWindowList(0)="DeusEx.DeusExRootWindow"
}
