//=============================================================================
// ANNAManager. (C) 2012 "Dei" & Nobody
//=============================================================================
class ANNAManager extends Actor config(ANNA);

var() config float MaxFrobDistance;
var() config bool bEnableSpectating;
var() config bool bResetSkin;

const ANNA_VERSION="101"; // Current version number. It's changed by every update.

function PostBeginPlay()
{
    local ANNAManager curManager;
    local bool bFound;

    foreach AllActors(class'ANNAManager', curManager)
      if(curManager != None && curManager != self)
          bFound = true;

    if(!bFound && ROLE == ROLE_Authority)
        setTimer(3, false);
    else
      Destroy();
}

/**Gets all information about the running game and starts
   a new game with the same conditions but with the miniMTL
   gametype.
   The function is called by PostBeginPlay()
*/
function Timer()
{
    local string currentMap;
    local class<GameInfo> currentGameClass,newGameClass;

    currentGameClass=level.game.class;
    currentMap = left(string(level),instr(string(level),"."));
    switch(currentGameClass)
    {
        case class'MTLDeathMatch': newGameClass=class'ANNADeathMatch'; break;
        case class'MTLTeam':       newGameClass=class'ANNATeam';       break;
        case class'MTLBasicTeam':  newGameClass=class'ANNABasicTeam';  break;
        case class'MTLAdvTeam':    newGameClass=class'ANNAAdvTeam';    break;
    }

    if(newGameClass != none)
    {
        log("Loading ANNA...",'ANNA');
        ConsoleCommand("servertravel"@currentMap$"?Game="$string(newGameClass));
    }
}

/**
 Server console kick.
*/
function serverKick(int playerID)
{
	local Pawn P;
  local string playerName;

  for(P=Level.PawnList; P!=None; P=P.NextPawn)
  {
    if(ANNAPlayer(P) != None && P.PlayerReplicationInfo != None)
    {
      if(P.PlayerReplicationInfo.PlayerID == playerID)
      {
        playerName = P.PlayerReplicationInfo.PlayerName@"("$P.PlayerReplicationInfo.PlayerID$")";
        ANNAPlayer(P).V7D = true;
        P.Destroy();
        Log(playername@"has been kicked.", 'ANNA');
        return;
      }
    }
  }
  Log("Could not find a player with ID"@playerID, 'ANNA');
}

/**
 Server console ban.
*/
function serverBan(int playerID) 
{
	local Pawn P;
	local string IP, playerName;
	local int i;
  
  for(P=Level.PawnList; P!=None; P=P.NextPawn)
  {
    if(ANNAPlayer(P) != None && P.PlayerReplicationInfo != None && NetConnection(PlayerPawn(P).Player) != None)
    {
      if(P.PlayerReplicationInfo.PlayerID == playerID)
      {
        IP = ANNAPlayer(P).GetPlayerNetworkAddress();
        if(Level.Game.CheckIPPolicy(IP))
        {
          playerName = P.PlayerReplicationInfo.PlayerName@"("$P.PlayerReplicationInfo.PlayerID$")";
          IP = Left(IP, InStr(IP, ":"));
          Log("Adding IP Ban for: "$IP, 'ANNA');
          for(i=0; i<50; i++)
            if(Level.Game.IPPolicies[i] == "")
              break;

          if(i < 50)
            Level.Game.IPPolicies[i] = "DENY,"$IP;
          Level.Game.SaveConfig();
        }
        P.Destroy();
        Log(playername@"has been banned & kicked.", 'ANNA');
        return;
      }
    }
  }
  Log("Could not find a player with ID"@playerID, 'ANNA');
}

function serverPlayerList()
{
	local Pawn P;
  local string IP;

  Log("	ID		PLAYERNAME		IP-ADDRESS", 'ANNA');
  for(P=Level.PawnList; P!=None; P=P.NextPawn)
  {
    if(ANNAPlayer(P) != None && P.PlayerReplicationInfo != None)
    {
      IP = ANNAPlayer(P).GetPlayerNetworkAddress();
      IP = Left(IP, InStr(IP, ":"));
      Log("	"$P.PlayerReplicationInfo.PlayerID$"		"$P.PlayerReplicationInfo.PlayerName$"		"$IP, 'ANNA');
    }
  }
}

/**
 Generates a 16 byte long Hexa-Decimal Unique ID.
*/
static function string generateUID()
{
  local int i;
  local string UID;

  for(i=0; i<16; i++)
  {
    if(FRand() < 0.5)
      UID = UID$string(Rand(9));
    else
      UID = UID$Chr(65+Rand(5));
  }
  return UID;
}


defaultproperties
{
   MaxFrobDistance=112.00
   bEnableSpectating=true
   bResetSkin=true
   bHidden=true
}
