//=============================================================================
// ANNAPlayer. (C) 2012 "Dei" & Nobody
//=============================================================================
class ANNAPlayer extends MTLPlayer abstract;

var float defaultMaxFrobDistance;
var bool bSpectatingEnabled;
var ANNAManager Manager;
var float spectateStartTime;

replication
{
    reliable if(ROLE == ROLE_Authority)
      clientStopFiring, clientUpdateURL, defaultMaxFrobDistance, bSpectatingEnabled;
      
	reliable if (Role < ROLE_Authority)
		Spectate;
}

/**
  Spawning the AntiCheat actor.
*/
function PostBeginPlay()
{
    local ANNAManager Manager;

    Super.PostBeginPlay();

    Spawn(class'ANNAAntiCheat', self);

    if(Manager == None)
      Manager = findManager();

    if(Manager != None)
      defaultMaxFrobDistance = Manager.MaxFrobDistance;
}

/**
  Find and return the ANNAManager
*/
function ANNAManager findManager()
{
  local ANNAManager curManager;
  local bool bFound;

  if(Manager != None)
    return Manager;

  foreach AllActors(class'ANNAManager', curManager)
    if(curManager != None)
      return curManager;

  return none;
}

/**
  Before we actually frob our target, reset the MaxFrobDistance.
*/
function DoFrob(Actor Frobber, Inventory frobWith)
{
  HighlightCenterObject();
  Super.DoFrob(Frobber, frobWith);
}

/**
  Reset the MaxFrobDistance
*/
function HighlightCenterObject()
{
  MaxFrobDistance = defaultMaxFrobDistance;
  Super.HighlightCenterObject();
}

/**
  Make sure the player uses our customized AugmentationManager 
*/
function InitializeSubSystems()
{
	local Skill VAC;
  
  if(Manager == None)
    Manager = findManager();

  bSpectatingEnabled = Manager.bEnableSpectating;

	if((Level.NetMode == NM_Standalone) && (BarkManager == None))
		BarkManager = Spawn(Class'BarkManager', self);

	CreateColorThemeManager();
	if(ThemeManager != None)
		ThemeManager.SetOwner(self);

	if(Level.NetMode != NM_Standalone)
	{
		if((AugmentationSystem != None) && !AugmentationSystem.IsA('ANNAAugmentationManager'))
		{
			AugmentationSystem.ResetAugmentations();
			AugmentationSystem.Destroy();
			AugmentationSystem = None;
		}

		if((SkillSystem != None) && !SkillSystem.IsA('CBPSkillManager'))
		{
			VAC = SkillSystem.FirstSkill;

			while(VAC != None)
			{
				VAC.Destroy();
				VAC = VAC.Next;
			}
			SkillSystem.Destroy();
			SkillSystem = None;
		}
	}

	if(AugmentationSystem == None)
	{
		AugmentationSystem = Spawn(Class'ANNAAugmentationManager', self);
		AugmentationSystem.CreateAugmentations(self);
		AugmentationSystem.AddDefaultAugmentations();
	}
  else
  {
		AugmentationSystem.SetPlayer(self);
		AugmentationSystem.SetOwner(self);
	}

	if(SkillSystem == None)
	{
		SkillSystem = Spawn(Class'CBPSkillManager',self);
		SkillSystem.CreateSkills(self);
	}
  else
  {
		SkillSystem.SetPlayer(self);
		SkillSystem.SetOwner(self);
	}

	if((Level.NetMode == NM_Standalone) || !bBeltIsMPInventory)
		CreateKeyRing();
}

/**
  Make sure the player uses our customized HUD 
*/
event Possess()
{
	local DeusExRootWindow root;

	Super.Possess();

	root = DeusExRootWindow(RootWindow);
	if(root != None)
	{
		if(root.HUD != None)
			root.HUD.Destroy();

		root.HUD = ANNAHUD(root.NewChild(Class'ANNAHUD'));
		root.HUD.UpdateSettings(self);
		root.HUD.SetWindowAlignments(HALIGN_Full, VALIGN_Full, 0.00, 0.00);
	}
}

/**
  Make sure the player uses our new scoreboard for the multiplayer-win screen.
*/
function ShowMultiplayerWin(String winnerName, int winningTeam, String Killer, String Killee, String Method)
{
	local ANNAHUDMP mpScr;
	local DeusExRootWindow root;

	if((Player != None) && (Player.Console != None))
		Player.Console.ClearMessages();

	root = DeusExRootWindow(rootWindow);

	if(root != None)
	{
		mpScr = ANNAHUDMP(root.InvokeUIScreen(Class'ANNAHUDMP', True));
		root.MaskBackground(True);

		if(mpScr != None)
		{
      mpScr.winnerName = winnerName;
      mpScr.winningTeam = winningTeam;
      mpScr.winKiller = Killer;
      mpScr.winKillee = Killee;
      mpScr.winMethod = Method;
		}
	}

   if(PlayerIsClient())
      if(AugmentationSystem != None)
         AugmentationSystem.DeactivateAll();
}

/**
  New <i>server -> client</i> function to tell the client to stop with firing.
  In this case the client is supposed to stop, if the player jumps into the water.
*/
simulated function clientStopFiring()
{
  DeusExWeapon(inHand).GotoState('SimFinishFire');
  DeusExWeapon(inHand).PlayIdleAnim();
}

simulated function clientUpdateURL(string NewOption, string NewValue, bool bSaveDefault)
{
  UpdateURL(NewOption, NewValue, bSaveDefault);
}

/**
  Extended state swimming for fixing the water bug.
*/
state PlayerSwimming
{
    /**
      Checks if the weapon is able to work under water, if not it forces the client to stop with firing.
    */
    function BeginState()
    {
        local Class<ThrownProjectile> thrownProjectileClass;
        local bool bOverride;
        bOverride = false;

        Super.BeginState();
        if(DeusExWeapon(inHand) != None)
        {
            if(DeusExWeapon(inHand).ProjectileClass != None)
            {
                thrownProjectileClass = Class<ThrownProjectile>(DeusExWeapon(inHand).ProjectileClass);
                if(thrownProjectileClass != None)
                  bOverride = true;
            }
            if(((DeusExWeapon(inHand).EnviroEffective == ENVEFF_Air) || (DeusExWeapon(inHand).EnviroEffective == ENVEFF_Vacuum) || (DeusExWeapon(inHand).EnviroEffective == ENVEFF_AirVacuum)) && !bOverride)
            {
               DeusExWeapon(inHand).GotoState('FinishFire');
               clientStopFiring();
            }
        }
    }

    /**
      Modified fire command-function which just makes the weapon firing, if it's able to fire under water.
      So that the player caN't try to continue firing under water if he presses the fire button again.
    */
    exec function Fire(optional float F)
    {
        local Class<ThrownProjectile> thrownProjectileClass;
        local bool bOverride;
        bOverride = false;

        if(DeusExWeapon(inHand) != None)
        {
          if(DeusExWeapon(inHand).ProjectileClass != None)
          {
              thrownProjectileClass = Class<ThrownProjectile>(DeusExWeapon(inHand).ProjectileClass);
              if(thrownProjectileClass != None)
                bOverride = true;
          }

          if(((DeusExWeapon(inHand).EnviroEffective == ENVEFF_Air) || (DeusExWeapon(inHand).EnviroEffective == ENVEFF_Vacuum) || (DeusExWeapon(inHand).EnviroEffective == ENVEFF_AirVacuum)) && !bOverride)
            return;
        }

        Super.Fire(f);
    }
}

/**
  Only make PopHealth available in singleplayer games. In multiplayer it could be abused for 'god mode'.
*/
function PopHealth(float health0, float health1, float health2, float health3, float health4, float health5)
{
  if(Level.NetMode == NM_StandAlone)
    Super.PopHealth(health0, health1, health2, health3, health4, health5);
}

/**
  Only make ServerReStartGame available in singleplayer games. In multiplayer it could crash the server.
*/
function ServerReStartGame()
{
  if(Level.NetMode == NM_StandAlone)
	  Level.Game.RestartGame();
}

/**
  MaintainEnergy could generate energy when a negative deltaTime was given.
*/
function MaintainEnergy(float deltaTime)
{
  if(deltaTime > 0)
    Super.MaintainEnergy(deltaTime);
}

/**
  Most difficult and complex anti cheat. Disable the editactor command.
*/
exec function EditActor();

/**
  Disable ShowPath command, can be used to create log-spam & serverlag.
*/
exec function ShowPath();

/**
  Check if the spydrone augmentation is actually active.
*/
function CreateDrone()
{
  if(bSpyDroneActive)
    Super.CreateDrone();
}

/**
  Make sure the item we drop is actually ours.
*/
exec function bool DropItem(optional Inventory inv, optional bool bDrop)
{
  if(inv != None && inv.Owner != self)
    return false;

  return Super.DropItem(inv, bDrop);
}

/**
  Make sure the player uses our main menu
*/
exec function ShowMainMenu()
{
	local DeusExRootWindow root;
	local DeusExLevelInfo info;
	local MissionEndgame Script;

	if(bIgnoreNextShowMenu)
	{
		bIgnoreNextShowMenu = False;
		return;
	}

	if(Level.NetMode == NM_Standalone)
    info = GetLevelInfo();

	ConsoleCommand("FLUSH");

	if((info != None) && (info.MissionNumber == 98)) 
	{
		bIgnoreNextShowMenu = True;
		PostIntro();
	}
	else if((info != None) && (info.MissionNumber == 99))
	{
		foreach AllActors(class'MissionEndgame', Script)
			break;

		if(Script != None)
			Script.FinishCinematic();
	}
	else
	{
		root = DeusExRootWindow(rootWindow);
		if(root != None)
			root.InvokeMenu(Class'ANNAMenuMain');
	}
}

/**
  Check and (re-)set skin. Only applies to team based games.
*/
simulated function CheckSkin()
{
    if(PlayerReplicationInfo != none)
    {
        if(PlayerReplicationInfo.Team == 0) // TEAM_UNATCO
        {
            Mesh = LodMesh'MPCharacters.mp_jumpsuit';
            DrawScale = 0.93;
            MultiSkins[0] = Texture'DeusExCharacters.Skins.MiscTex1';
            MultiSkins[1] = Texture'DeusExCharacters.Skins.UNATCOTroopTex1';
            MultiSkins[2] = Texture'DeusExCharacters.Skins.UNATCOTroopTex2';
            MultiSkins[3] = Texture'DeusExCharacters.Skins.MiscTex1';
            MultiSkins[4] = Texture'DeusExCharacters.Skins.MiscTex1';
            MultiSkins[5] = Texture'DeusExItems.Skins.PinkMaskTex';
            MultiSkins[6] = Texture'DeusExCharacters.Skins.UNATCOTroopTex3';
            MultiSkins[7] = Texture'DeusExItems.Skins.PinkMaskTex';
        }
        else if(PlayerReplicationInfo.Team == 1) // TEAM_NSF
        {
            Mesh = LodMesh'MPCharacters.mp_jumpsuit';
            DrawScale = 0.93;
            MultiSkins[0] = Texture'DeusExCharacters.Skins.TerroristTex0';
            MultiSkins[1] = Texture'DeusExCharacters.Skins.TerroristTex2';
            MultiSkins[2] = Texture'DeusExCharacters.Skins.TerroristTex1';
            MultiSkins[3] = Texture'DeusExCharacters.Skins.TerroristTex0';
            MultiSkins[4] = Texture'DeusExCharacters.Skins.TerroristTex0';
            MultiSkins[5] = Texture'DeusExItems.Skins.PinkMaskTex';
            MultiSkins[6] = Texture'DeusExCharacters.Skins.GogglesTex1';
            MultiSkins[7] = Texture'DeusExItems.Skins.PinkMaskTex';
        }
        else if(Level.Game.bTeamGame) //if no valid team ID found and if it's a teamgame, then disconnect the player
        {
            Log("Invalid Team ID ("$PlayerReplicationInfo.Team$")... Disconnecting player ("$PlayerReplicationInfo.PlayerName$").",'ANNA');
            ForceDisconnect("Invalid Team ID detected");
        }
        V7C = false;
    }
}

/**
  Check & fix inventory: projectiles don't have an owner when the player crashes and returns.
*/
function CheckInventory()
{
	local Inventory Inv;
  
  if(Inventory != None)
    for(Inv = Inventory; Inv != None; Inv = Inv.Inventory)
      Inv.Instigator = self;
}

/**
  Added support for teamsay messages in the MessagingSpectator.
*/
exec function TeamSay (string Z38)
{
	local Pawn P, Player;
	local string str;

  if(V7D)
		return;

	if(TeamDMGame(DXGame) == None)
	{
		Say(Z38);
		return;
	}

	if(!PlayerIsListenClient() && (Level.NetMode != NM_Standalone))
	{
		V62(Len(Z38));
		if(V4A())
			return;
	}

	if (Z38 != "")
		V54(Z38,True);

	if(Z38 == "")
		return;

	str = V50() $ ":" @ Z38;
	if(Role == 4)
		Log(str, 'TeamSay');

	for(P=Level.PawnList; P!=None; P=P.NextPawn)
	{
		if((P.IsA('MTLPlayer') && (P.PlayerReplicationInfo.Team == PlayerReplicationInfo.Team)))
		{
			P.ClientMessage(str, 'TeamSay', True);
		}
    else if(P.IsA('MessagingSpectator'))
    {
      if(PlayerReplicationInfo.Team == 0) // TEAM_UNATCO
			  P.ClientMessage(str,'TeamUNATCOSay', True);
      else if(PlayerReplicationInfo.Team == 1) // TEAM_NSF
			  P.ClientMessage(str,'TeamNSFSay', True);
    }
	}
}

exec function Spectate()
{
  if(Manager == None)
    Manager = findManager();

  if(Manager.bEnableSpectating)
  {
    if((spectateStartTime+5) < Level.TimeSeconds)
    {
      spectateStartTime = Level.TimeSeconds;
      GotoState('PlayerSpectating');
      BroadcastMessage(PlayerReplicationInfo.PlayerName@"started spectating.");
    }
  }
}

state PlayerSpectating
{
  ignores SeePlayer, HearNoise, Bump, TakeDamage, Died, ZoneChange, FootZoneChange, ActivateAugmentation, BuySkills;

	exec function Suicide();
	exec function AltFire(optional float F);
	exec function God();
	exec function BehindView(bool B);
  exec function Invisible(bool B);
  exec function Ghost();
  exec function Walk();
  exec function Fly();

  exec function TeamSay(string Z38)
  {
    Say(Z38);
  }

	exec function Fire(optional float F)
	{
	}

  function ChangeTeam( int N )
  {
    Level.Game.ChangeTeam(self, N);
  }
  
  exec function Spectate()
  {
		ReducedDamageType = '';
    if(Level.Game.bTeamGame)
    {
      if(TeamDMGame(Level.Game) != None)
        PlayerReplicationInfo.Team = TeamDMGame(Level.Game).GetAutoTeam();

      if(PlayerReplicationInfo.Team == 0) // TEAM_UNATCO
        BroadcastMessage(PlayerReplicationInfo.PlayerName@"joined UNATCO.");
      else if(PlayerReplicationInfo.Team == 1) // TEAM_NSF
        BroadcastMessage(PlayerReplicationInfo.PlayerName@"joined NSF.");
      else // TEAM_UHH?
        BroadcastMessage(PlayerReplicationInfo.PlayerName@"started playing.");
    }
    else
    {
      PlayerReplicationInfo.Team = 128;
      BroadcastMessage(PlayerReplicationInfo.PlayerName@"started playing.");
    }
    clientUpdateURL("Team", string(PlayerReplicationInfo.Team), true);
    V7C = false;
    Level.Game.RestartPlayer(self);
    GotoState('PlayerWalking');
  }

	function SendVoiceMessage(PlayerReplicationInfo Sender, PlayerReplicationInfo Recipient, name messagetype, byte messageID, name broadcasttype)
	{
	}

  exec function ParseLeftClick()
  {
  }
  
  exec function ParseRightClick()
  {
  }
  
  function bool RestrictInput()
  {
    return true;
  }
	
	function ProcessMove(float DeltaTime, vector NewAccel, eDodgeDir DodgeMove, rotator DeltaRot)	
	{
		Acceleration = Normal(NewAccel);
		Velocity = Normal(NewAccel) * 300;
		AutonomousPhysics(DeltaTime);
	}

	event PlayerTick(float DeltaTime)
	{
		RefreshSystems(DeltaTime);
		UpdateTimePlayed(DeltaTime);
		if(bUpdatePosition)
			ClientUpdatePosition();

		PlayerMove(DeltaTime);
	}

	function PlayerMove(float DeltaTime)
	{
		local rotator newRotation;
		local vector X,Y,Z;

		GetAxes(ViewRotation,X,Y,Z);

		aForward *= 0.1;
		aStrafe  *= 0.1;
		aLookup  *= 0.24;
		aTurn    *= 0.24;
		aUp		 *= 0.1;
	
		Acceleration = aForward*X + aStrafe*Y + aUp*vect(0,0,1);  

		UpdateRotation(DeltaTime, 1);

		if ( Role < ROLE_Authority ) // then save this move and replicate it
			ReplicateMove(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
		else
			ProcessMove(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
	}

	function EndState()
	{
    bVisionImportant = true;
    ResetBasedPawnSize();
    PlayerReplicationInfo.Score = 0.0;
    PlayerReplicationInfo.Deaths = 0.0;
    PlayerReplicationInfo.Streak = 0.0;
		PlayerReplicationInfo.bIsSpectator = false;
		SkillPointsAvail = 0;
		SkillPointsTotal = 0;
    bDetectable = true;
    bHidden = false;
		SetCollision(true,true,true);
	}

	function spectateInitialize()
	{
		local SecurityCamera camera;
		local AutoTurret turret;
		local DeusExDecoration decoration;
    local ThrownProjectile projectile;

		foreach AllActors(Class'SecurityCamera',camera)
		{
      if(camera.safeTarget == self)
      {
        camera.safeTarget = None;
        camera.Team = -1;
      }
    }
    
		foreach AllActors(Class'AutoTurret', turret)
		{
      if(turret.safeTarget == self)
      {
        turret.safeTarget = None;
        turret.Team = -1;
      }
    }

		foreach AllActors(Class'ThrownProjectile', projectile)
		{
      if(projectile.Owner == self)
      {
        projectile.bDisabled = true;
        projectile.Team = -1;
      }
    }
    
		foreach AllActors(Class'DeusExDecoration', decoration)
		{
      if(decoration.Owner == self)
      {
        decoration.SetOwner(None);
      }
    }
	}

	function BeginState()
	{
    local Inventory Inv;
	  local DeusExWeapon W;
    
    W = DeusExWeapon(Weapon);
    if(W != None)
      W.ScopeOff();

    DropDecoration();
		if(inHand != None)
	    SetInHand(None);

    spectateInitialize();

    if(Inventory != None)
    {
      for(Inv = Inventory; Inv != None; Inv = Inv.Inventory)
      {
        DeleteInventory(Inv);
        Inv.Destroy();
      }
    }

		if(invulnSph != None)
		{
			invulnSph.Destroy();
			invulnSph = None;
		}

    if(AugmentationSystem != None)
      AugmentationSystem.DeactivateAll();

    DeusExRootWindow(RootWindow).HUD.belt.ClearBelt();

		bNintendoImmunity = False;
		NintendoImmunityTimeLeft = 0.0;
    ViewTarget = None;
    KillShadow();

    bBehindView = False;
    FrobTarget = None;
	  ReducedDamageType = 'All';
    bVisionImportant = false;
    SetBasedPawnSize(Default.CollisionRadius, 30.0);
    PlayerReplicationInfo.Score = 0.0;
    PlayerReplicationInfo.Deaths = 0.0;
    PlayerReplicationInfo.Streak = 0.0;
    PlayerReplicationInfo.bIsSpectator = true;
    PlayerReplicationInfo.Team = 100;
		SkillPointsAvail = 0;
		SkillPointsTotal = 0;
    bDetectable = false;
    bHidden = true;
		SetCollision(false,false,false);
    bCollideWorld = true;
		SetPhysics(PHYS_Flying);
	}
}

function ForceDisconnect(string Message)
{
  if(Player != None && Player.Console != None)
	  Player.Console.AddString(Message);

	DisconnectPlayer();
}

DefaultProperties
{
  defaultMaxFrobDistance=112.00
}
